
# MASQ Web App

## What is the MASQ Mobile app? 

You can find some helpful guidance on our website [here](www.tseboston.com/masq). The MASQ app was inspired by the current workflow for sizing military respirators is time consuming, manually intensive, and tedious, but is necessary due to the critical need for respirators to fit properly, especially in operating environments when Warfighters may be exposed to chemical, biological, radiological, and nuclear (CBRN) threats. To improve the fitting process, TSE developed the Mask Analysis and Size Quantification (MASQ) tool to rapidly generate a 3D model of a user’s face, accurately compute anthropometric measurements, and estimate the appropriate size of a respiratory protective mask.

## The Web App

The MASQ Web App allows a user to view the 3D Head forms and associated biographical data that are uploaded from the MASQ Mobile App. Within the web app the user can filter their results based on  anthropometric data.

You can check out an example of MASQ web app [here](masqmobileapp.com).

## About this README
Please use this document as a guide; knowing that some of the information, advice and guidelines etc. may not work in your environment or could potentially be out of date. 

This guide does not account for best security practices. 

# Setting up the AWS Backend
You will need an [S3 bucket](https://docs.aws.amazon.com/AmazonS3/latest/user-guide/create-bucket.html). You will also need to setup an [IAM](https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_access-keys.html) user with access to the DynamoDB and S3 bucket. The S3 bucket is used to store all of the 3D head forms and it is used in conjunction with the [DynamoDB](https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/getting-started-step-1.html) which will store all measurement data. 

## The DynamoDB
1. Create a table with a table name of your choosing, or use our default "MasqMeasurements" 
1. Set the primary key to "MeasurementID" as a string
1. Table settings can be left with defaults
1. Column names will be populated when data is uploaded. 

## IAM Setup 
1. Create a new user with a username of your choosing or use our default "MASQ" 
1. Access type should be "Programmatic access" as you will need to get an access key
1. **Permissions should be customized to your environment**. We assume that you will have full read/write access to the S3 bucket you created earlier and you give full access to the table that you created in DynamoDB. 
    - The **not** secure way is to allow the following polcies: AmazonS3FullAccess and AmazonDynamoDBFullAccesswithDataPipeline. 
1. You do not need to create any tags. 
1. Now is a good time to copy and past the access IDs inside the aws.json file. 

# Locally Setting Up and Running for the First Time
1. Download and install npm (node.js) [here is the download page for node](https://nodejs.org/en/download/). You can verify the install worked by opening a terminal/cmd and typing ```npm```

1. Open a terminal and change into the project directory (\src\masq_web_app\) and run ```npm install```. This will download all dependencies/packages etc. required for the app

1. You can then run the app by typing ```npm start```. By default the app will run in your browser at [http://localhost:3000/](http://localhost:3000/) or [your-device-ip-here]:3000

### Running The App Locally

4. To run the app again follow step 3 from above

# Guidance on Hosting

Below are some resources that we have written to help guide you in the setup process. Please note that these instructions are up to date as of writing but services change over time.

- If you would like to read more about Deploying React resrouces you can read [React Production Build](https://create-react-app.dev/docs/production-build), [React Deployment](https://create-react-app.dev/docs/deployment)


## Before building you will need to
Add your host keys to the ```aws.json``` file within the ```\src``` directory. For example: 

        "awsParams": {
            "region": "us-east-1",
            "credentials": {
                "accessKeyId": "YOUR_ACCESS_KEY_ID",
                "secretAccessKey": "YOUR_SECRET_ACCESS_KEY"
             }
You will also need to specify the S3 service and DynamoDB services. For example

```
    "dynamoParams": {
        "tableName": "MasqMeasurements",
        "apiVersion": "2012-08-10"
    },
    "s3Params": {
        "bucketName": "masq-models",
        "apiVersion": "2006-03-01"
    }

```

## Building the React app for production 
Before starting make sure you have installed npm, added your AWS keys to the JSON file and setup the AWS backend. In this directory you can run ```npm run build``` in a terminal.  This spits out a build directory housing everything you need for deployment. You can then test the build with ```serve -s build```. This will launch the web app. If everything is working you should be able to view the app and any 3D models stored within AWS. 

## Hosting
You can host anywhere that allows a node server to run. Some hosting sights include Amazon Light Sail, Google Firebase, or just a regular Ubuntu server. Throughout this README we use the example of hosting with Bitnami (who provides a node server with some cloud providers) on a Linux machine using Amazon Light Sail. However, you can use another service to host but the service will need to allow for the use of React-Router (meaning static servers may not works as expected). 

# Deployment on Amazon Light Sail with Bitnami's Node Service
We have our instance of the MASQ Web App hosted on [Amazon Light Sail](https://aws.amazon.com/lightsail/) using a Bitnami Node server. 

## Steps
1. Setup the Apache Web Server
2. Run the web app 

### General Information
- The local user is bitnami
- To get the password for the local user, cd into the home directory and type cat bitnami_application_password and the password for the user will be displayed on the screen
- The directory for the Apache web server is ```/opt/bitnami/apache/``
- The directory for the react app is /opt/masq-web-app/masq-web-app

## Apache and the React App
Apache is the web server used to proxy all incoming web traffic to the React app running on port 5000 (or a port of your choosing). You can find some of this information in the apache http configuration docs. 

### The Apache config files
Check the Apache directory mentioned above for the http configuration. These files contain the proxy port for the React app and the SLL configuration settings. The current port for the React server app is 5000.You can read a guide [here](https://docs.bitnami.com/aws/infrastructure/nodejs/get-started/get-started/#step-3-serve-your-application-through-the-apache-web-server) on setting up a custom configuration with Apache. 

## Starting the React app with a Systemd Service and a Shell Script

All scripts mentioned here are in the ./server-resources directory of this repo.

There is a service Daemon that starts only the react app (Apache is restarted through Bitnami) on restart. The service file is located in ```/etc/systemd/system/masq-react-app.service``` and points to a shell script, ```run_web_app_forever.sh```.

The shell script to start the React app is located here:
```/path/to/location/of/run_web_app_forever.sh ```
It specifies the correct directories + commands to run ```node -s serve```. 
 
To check the status/start/restart etc. the React app run this command with the correct options:
```sudo systemctl [start | restart | stop | status] masq-react-app.service```

Making changes to the service file requiers you to run ```sudo systemctl reenable```. 

## Certificates for Hosting 
[Let's Encrypt](https://letsencrypt.org/) and certbot provide a tool for free and reliable SSL certificate, they will also auto renew if you enable them to. 

If using a Bitnami implementation you can use their built in tool with their Apache configuration, more information [here](https://docs.bitnami.com/aws/faq/administration/generate-configure-certificate-letsencrypt/)

### Deploying the build on the server
If you decide to go with a more standard server implementation you will need to SFTP into the server (a tool like [Filezilla](https://filezilla-project.org/) **client** will work). Next head to the following directory ```/path/to/masq-web-app/``` and replace the current build directory with the contents of the new build directory (directions from above). You do not need to restart the service but its best you do, use the following command to do so ```sudo systemctl restart masq-react-app```.