import React, { useState, useEffect, useRef } from 'react'
import * as THREE from 'three'
// import OrbitControls from 'three-orbitcontrols'
import { ObjectControls } from '../ObjectControls'
import {MTLLoader, OBJLoader} from 'three-obj-mtl-loader'

import db from '../db'
import { findAllByDisplayValue } from '@testing-library/react'

export default function ModelViewer(props) {
  const [modelData, setModelData] = useState(null)
  const [renderer, setRenderer] = useState(null)
  const [scene, setScene] = useState(null)
  const [camera, setCamera] = useState(null)
  //Display an error/message to the user if needed
  const [message, setMessage] = useState(null)
  const mountRef = useRef(null)

  useEffect(() => {
    const width = mountRef.current.clientWidth;
    const height = mountRef.current.clientHeight;

    var renderer = new THREE.WebGLRenderer({ antialias: true, alpha: true });
    renderer.setPixelRatio(window.devicePixelRatio * 2);
    renderer.setSize(width, height);
    renderer.setClearColor('#E2E8F0');
    setRenderer(renderer)
    
    mountRef.current.appendChild(renderer.domElement);

    var scene = new THREE.Scene();
    setScene(scene)

    //ADD CAMERA
    var camera = new THREE.PerspectiveCamera(75, width / height, 1, 10000);
    camera.position.z = 200;
    setCamera(camera)

    // ADD LIGHT
    var light = new THREE.PointLight( 0xffffff, 0.9, 0 );
    light.position.set(0, 0, 50);
    scene.add(light);

    camera.lookAt(scene.position)

    const animate = function() {
      requestAnimationFrame(animate)
      // controls.update()
      renderer.render(scene, camera)
    }
    animate()

    //ADD CONTROLS
    // var controls = new OrbitControls(camera);
  }, [])

  useEffect(() => {
    setMessage(null);
    if (props.modelId) {      
      db.getModelObjects(props.modelId) 
      .then(data => {
        console.log('setting model data to', data)
        setModelData(data)
      })
      .catch(err => { 
        console.log("Received error from server when trying to fetch model ",err );
        setMessage("Unable to load 3D model"); 
        setModelData(0);
      })
    }
    else {
      setModelData(null)
    }
  }, [props.modelId])

  useEffect(() => {
    // Create a 3D text model to display to the user that the model did not load
    if (!modelData) {        
      console.log("Could not load the model data into the viewer")
      return;
    }
    console.log('rendering model')

    //ADD MASK
    var manager = new THREE.LoadingManager();
    // let mtlLoader = new MTLLoader(manager);
    let objLoader = new OBJLoader(manager);
    let texLoader = new THREE.TextureLoader(manager);

    //GENERATE MODEL
    console.log('loading texture from:', modelData.isomapUrl)
    // texLoader.setCrossOrigin('anonymous')
    const texture = texLoader.load(modelData.isomapUrl);
    const texMaterial = new THREE.MeshPhongMaterial({map: texture});
    // mtlLoader.load(modelData.materialUrl, (materials) => {
        // materials.preload();
        // objLoader.setMaterials(materials);

    let modelObject = null
    objLoader.load(modelData.modelUrl, (object) => {
      modelObject = object
      modelObject.traverse(function(child) {
          if (child instanceof THREE.Mesh) {
              child.material = texMaterial;
          }
      })
      scene.add(modelObject)
      console.log(modelObject.children[0])
      var controls = new ObjectControls(camera, renderer.domElement, modelObject.children[0])
      controls.setDistance(8, 200); // set min - max distance for zoom
      controls.setZoomSpeed(4.0); // set zoom speed
      controls.enableVerticalRotation();
      controls.setMaxVerticalRotationAngle(Math.PI / 4, Math.PI / 4);
      controls.setRotationSpeed(0.05);

      // controls.setObjectToMove()
    })
    // var controls = new THREE.ObjectControls(camera, renderer.domElement, modelObject)
    // controls.update()
    // })

    // camera.position.x += (mouseX - camera.position.x) * .05;
    // camera.position.y += (-mouseY - camera.position.y) * .05;
    
    return () => {
      scene.remove(modelObject)
    }
  }, [renderer, scene, camera, modelData])

  if(message) {
    console.log("Inside first condition")
  return (
    <div className={props.className}>
      <div></div>
      <p></p>
      <div className="h-full text-center text-red-600" ref={mountRef}>
        {message}
      </div>
    </div>
  )
  } else { 
    console.log("Inside second condition")
    return (
    <div className={props.className}>
      <div className="h-full" ref={mountRef}>
      </div>
    </div>
    )
  }
}